#!/bin/sh
#
# Copyright (c) 2006, University of Karlstad
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:

# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

# $Id: install.sh,v 1.9 2006/06/08 13:12:04 medea3 Exp $

# Prgs
alias echo='echo -e'
MYSQL_PRG="mysql"

# Variables

# System
SYSTEM_NAME="zeus"
SYSTEM_PATH="/zeus"
INSTALL_CACHE=".install"
SETTINGS_FILE="settings.php"
SQL_TABLE_STRUCTURE="database/zeus_table_structure.sql"
SQL_UPDATE_FILE="update_database.sql"
MY_HOSTNAME=`hostname`

# Read from install cache
DB_HOST="localhost"
DB_NAME="zeus"
ENABLE_SMTP="n"
FIRST="1"
INSTALL_ROOT="/tmp"
ADMIN_URL="http://"$MY_HOSTNAME"/admin"
USER_URL="http://"$MY_HOSTNAME"/"

if [ -e $INSTALL_CACHE ]; then
	. .install
	FIRST="0"
fi

# Functions
cleanup()
{
	rm $SETTINGS_FILE $SQL_UPDATE_FILE 2>/dev/null 1>/dev/null
	EXIT_STATUS=`echo $?`
	if [ "$EXIT_STATUS" != "0" ]; then
		echo "\nInstallation error: Couldn't remove setup files!"
		echo "Remove the files manually!"
	fi
}

# Main output
echo "\nWelcome to Zeus questionnaire system!"
echo "====================================="
echo "This script will ask some questions about" 
echo "your system setup in order to create an" 
echo "initial configuration.\n\n"
echo -n "Press [ENTER] to begin "
read
echo "\nSite properties"
echo "===============\n"

echo -n "Site title [$SITE_TITLE]: "
read TMP
if [ "$TMP" != "" ] ; then 
	SITE_TITLE=$TMP
fi

echo -n "Organisation [$ORGANISATION]: "
read TMP
if [ "$TMP" != "" ] ; then 
	ORGANISATION=$TMP
fi

echo -n "Enable SMTP ("
if [ "$ENABLE_SMTP" == 'y' -o "$ENABLE_SMTP" == 'Y' ] ; then 
	echo -n "Y/n"
else echo -n "y/N"
fi
echo -n "): "

read TMP
if [ "$TMP" == 'y' -o "$TMP" == 'Y' ] ; then
	ENABLE_SMTP="y"
fi
if [ "$TMP" == 'n' -o "$TMP" == 'N' ] ; then
	ENABLE_SMTP="n"
fi

if [ "$ENABLE_SMTP" == "y" -o "$ENABLE_SMTP" == "Y" ] ; then
	echo -n "SMTP server[$SMTP_SERVER]: "
	read TMP
	if [ "$TMP" != "" ] ; then 
		SMTP_SERVER=$TMP
	fi
fi

echo -n "Administator site URL [$ADMIN_URL]: "
read TMP
if [ "$TMP" != "" ] ; then 
	ADMIN_URL=$TMP
fi

echo -n "User site URL [$USER_URL]: "
read TMP
if [ "$TMP" != "" ] ; then 
	USER_URL=$TMP
fi

echo -n "Site manager name [$MANAGER_NAME]: "
read TMP
if [ "$TMP" != "" ] ; then 
	MANAGER_NAME=$TMP
fi

echo -n "Site manager email [$MANAGER_EMAIL]: "
read TMP
if [ "$TMP" != "" ] ; then 
	MANAGER_EMAIL=$TMP
fi

echo "\nAdmin properties (admin)"
echo "========================\n"

echo -n "Administrator name [$ADMIN_NAME]: "
read TMP
if [ "$TMP" != "" ] ; then 
	ADMIN_NAME=$TMP
fi

echo -n "Administrator email [$ADMIN_EMAIL]: "
read TMP
if [ "$TMP" != "" ] ; then 
	ADMIN_EMAIL=$TMP
fi

echo -n "Password: "
STTY_SETTINGS=`stty -g`
stty -echo
read TMP
if [ "$TMP" != "" ] ; then 
	ADMIN_PASSWD=$TMP
fi
stty $STTY_SETTINGS

echo "\n\nDatabase settings"
echo "===================\n"

echo -n "Adress to the mysql-server (host[:port])[$DB_HOST]: "
read TMP
if [ "$TMP" != "" ] ; then 
	DB_HOST=$TMP
fi

if [ $FIRST == "1" ]; then
	DB_HOST_CONFIG=$DB_HOST
fi

echo -n "Database adress to put in the zeus config (usually the same as above) (host[:port])[$DB_HOST_CONFIG]: "
read TMP
if [ "$TMP" != "" ] ; then 
	DB_HOST_CONFIG=$TMP
fi

echo -n "Username [$DB_USERNAME]: "
read TMP
if [ "$TMP" != "" ] ; then 
	DB_USERNAME=$TMP
fi

echo -n "Password: "
STTY_SETTINGS=`stty -g`
stty -echo
read TMP
if [ "$TMP" != "" ] ; then 
	DB_PASSWD=$TMP
fi
stty $STTY_SETTINGS

echo -n "\nDatabase name [$DB_NAME]: "
read TMP
if [ "$TMP" != "" ] ; then 
	DB_NAME=$TMP
fi

echo "\nInstallation path"
echo "===================\n"

echo -n "Select installation root (note, the script will create the zeus folder) [$INSTALL_ROOT]: "
read TMP
if [ "$TMP" != "" ] ; then 
	INSTALL_ROOT=$TMP
fi

# Write to file
echo SITE_TITLE=\"$SITE_TITLE\" > $INSTALL_CACHE
echo ORGANISATION=\"$ORGANISATION\" >> $INSTALL_CACHE
echo ENABLE_SMTP=\"$ENABLE_SMTP\" >> $INSTALL_CACHE
echo SMTP_SERVER=\"$SMTP_SERVER\" >> $INSTALL_CACHE
echo MANAGER_NAME=\"$MANAGER_NAME\" >> $INSTALL_CACHE
echo MANAGER_EMAIL=\"$MANAGER_EMAIL\" >> $INSTALL_CACHE
echo DB_HOST=\"$DB_HOST\" >> $INSTALL_CACHE
echo DB_HOST_CONFIG=\"$DB_HOST_CONFIG\" >> $INSTALL_CACHE
echo DB_USERNAME=\"$DB_USERNAME\" >> $INSTALL_CACHE
echo DB_NAME=\"$DB_NAME\" >> $INSTALL_CACHE
echo ADMIN_NAME=\"$ADMIN_NAME\" >> $INSTALL_CACHE
echo ADMIN_EMAIL=\"$ADMIN_EMAIL\" >> $INSTALL_CACHE
echo INSTALL_ROOT=\"$INSTALL_ROOT\" >> $INSTALL_CACHE
echo ADMIN_URL=\"$ADMIN_URL\" >> $INSTALL_CACHE
echo USER_URL=\"$USER_URL\" >> $INSTALL_CACHE

echo "\nThe zeus system will be installed with the following settings:"
echo "* Install path: $INSTALL_ROOT$SYSTEM_PATH"
echo "* Site title: $SITE_TITLE"
echo "* Organisation: $ORGANISATION"
echo "* Enable SMTP: $ENABLE_SMTP"
if [ "$ENABLE_SMTP" == "y" -o "$ENABLE_SMTP" == "Y" ]; then
	echo "* SMTP server: $SMTP_SERVER"
fi
echo "* Admin URL: $ADMIN_URL"
echo "* User URL: $USER_URL"
echo "* Manager name: $MANAGER_NAME"
echo "* Manager email: $MANAGER_EMAIL"
echo "* Database host: $DB_HOST"
echo "* Zeus config database host: $DB_HOST_CONFIG"
echo "* Database name: $DB_NAME"
echo "* Database user: $DB_USERNAME"
echo "* Admin name: $ADMIN_NAME"
echo "* Admin email: $ADMIN_EMAIL"
echo "\n*** NOTE ***"
echo "Proceeding with the installation will REMOVE any previously saved settings and EMPTY the database!"
echo -n "\nPress [ENTER] to begin the installation or CTRL-C to abort "
read

if [ $FIRST == "0" ]; then
	echo "\nIt seems that you've already has run this installation script. All settings and the database entries will be lost. Are you really sure that you want to proceed?"
	echo -n "\nPress [ENTER] to begin the installation or CTRL-C to abort "
read

fi

# Check install path

#check if it's a dir, if not try to create it
if [ ! -d $INSTALL_ROOT ]; then
	mkdir -p $INSTALL_ROOT 2>/dev/null 1>/dev/null
	EXIT_STATUS=`echo $?`
	if [ "$EXIT_STATUS" != "0" ]; then
		echo "Installation error: Couldn't create installation directory!"
		exit -1
	fi
fi

# check if we have write permissions
if [ -d $INSTALL_ROOT -a  ! -w $INSTALL_ROOT ]; then
	echo "\nInstallation error: Couldn't install in the specified directory! Permission denied!"	
	exit -1;
fi

# check the full path
FULL_PATH="$INSTALL_ROOT$SYSTEM_PATH"

if [ "$SYSTEM_NAME" == "" ]; then
	echo "\nInstallation error: FATAL error!"	
	exit -1;
fi

if [ $FULL_PATH == "/" -o $FULL_PATH == "/usr" -o $FULL_PATH == "/etc" -o $FULL_PATH == "/sbin" ]; then
	echo "\nInstallation error: FATAL error!"	
	exit -1;
fi

# Generate settings file

echo "<?php\n/*" > $SETTINGS_FILE
cat LICENSE >> $SETTINGS_FILE 
echo "\n*/" >> $SETTINGS_FILE
echo "\n//Database settings\n" >> $SETTINGS_FILE
echo "//Serveradress (can be 'localhost')" >> $SETTINGS_FILE
echo "\$setting_sql_server = '$DB_HOST_CONFIG';" >> $SETTINGS_FILE

echo "//Username for the specified server" >> $SETTINGS_FILE
echo "\$setting_sql_user = '$DB_USERNAME';" >> $SETTINGS_FILE

echo "//Password for the specified server" >> $SETTINGS_FILE
echo "\$settings_sql_password = '$DB_PASSWD';" >> $SETTINGS_FILE

echo "//Database for this system" >> $SETTINGS_FILE
echo "\$settings_sql_database = '$DB_NAME';" >> $SETTINGS_FILE

echo "?>" >> $SETTINGS_FILE


# Generate sql inserts/updates

if [ "$ENABLE_SMTP" == "y" -o "$ENABLE_SMTP" == "Y" ]; then
	ENABLE_SMTP="1"
fi
if [ "$ENABLE_SMTP" == "n" -o "$ENABLE_SMTP" == "N" ]; then
	ENABLE_SMTP="0"
fi

echo "DELETE FROM site_properties;" > $SQL_UPDATE_FILE
echo "DELETE FROM admin;" >> $SQL_UPDATE_FILE

echo "INSERT INTO site_properties (id) VALUES(1);" >> $SQL_UPDATE_FILE
echo "INSERT INTO admin (id) VALUES(1);" >> $SQL_UPDATE_FILE

echo "UPDATE site_properties SET site_title = '$SITE_TITLE', site_manager = '$MANAGER_NAME', site_manager_email = '$MANAGER_EMAIL', site_smtp = '$SMTP_SERVER', site_send_publish_mail = '$ENABLE_SMTP', site_org = '$ORGANISATION', site_admin_site = '$ADMIN_URL', site_user_site = '$USER_URL' WHERE id=1;" >> $SQL_UPDATE_FILE

echo "UPDATE admin SET username = 'admin', admin_name = '$ADMIN_NAME', admin_email = '$ADMIN_EMAIL', password = MD5('$ADMIN_PASSWD'), superuser = "1" WHERE id=1;" >> $SQL_UPDATE_FILE


# Check correct database settings

$MYSQL_PRG --host=$DB_HOST --database=$DB_NAME --user=$DB_USERNAME --password=$DB_PASSWD < /dev/null 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" == "1" ]; then
	echo "\nAn error occured while trying to connect to database \"$DB_NAME\" on \"$DB_HOST\" with username \"$DB_USERNAME\". Please check your mysql configuration and make sure that the database is created and setup with proper permissions. Then re-run the installation script."
	cleanup
	exit -1
fi

# Proceed with database setup

#create table structure
$MYSQL_PRG --host=$DB_HOST --database=$DB_NAME --user=$DB_USERNAME --password=$DB_PASSWD < $SQL_TABLE_STRUCTURE 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" == "1" ]; then
	echo "\nAn error occured while trying to connect to database \"$DB_NAME\" on \"$DB_HOST\" with username \"$DB_USERNAME\". Please check your mysql configuration and make sure that the database is created and setup with proper permissions. Then re-run the installation script."
	cleanup
	exit -1
fi

#update tables
$MYSQL_PRG --host=$DB_HOST --database=$DB_NAME --user=$DB_USERNAME --password=$DB_PASSWD < $SQL_UPDATE_FILE 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" == "1" ]; then
	echo "\nAn error occured while trying to connect to database \"$DB_NAME\" on \"$DB_HOST\" with username \"$DB_USERNAME\". Please check your mysql configuration and make sure that the database is created and setup with proper permissions. Then re-run the installation script."
	cleanup
	exit -1
fi

#check if already exists
if [ -d $FULL_PATH ]; then
	echo "\nInstallation error: The zeus system is already installed!"
	echo "Please remove $FULL_PATH manually and re-run this script!"
	cleanup
	exit -1
fi

cp -r $SYSTEM_NAME $INSTALL_ROOT 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't copy files!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

# setup permissions

find $INSTALL_ROOT -type d -exec chmod 755 \{\} \; 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi
find $INSTALL_ROOT -type f -exec chmod 644 \{\} \; 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

mkdir $FULL_PATH/admin/exported_files 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't create exported files!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

chmod 777 $FULL_PATH/admin/exported_files 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions on exported files!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

touch $FULL_PATH/admin/logs/querys.log 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't create query.log!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi
chmod 777 $FULL_PATH/admin/logs/querys.log 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions on querys.log"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi
cp $SETTINGS_FILE $FULL_PATH/admin/inc/settings/database.set.php 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't copy settings file!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi
chmod 644 $SETTINGS_FILE $FULL_PATH/admin/inc/settings/database.set.php 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions on the settings file!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

cp $SETTINGS_FILE $FULL_PATH/inc/settings/database.set.php 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't copy settings file!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

chmod 644 $SETTINGS_FILE $FULL_PATH/inc/settings/database.set.php 2>/dev/null 1>/dev/null
EXIT_STATUS=`echo $?`
if [ "$EXIT_STATUS" != "0" ]; then
	echo "\nInstallation error: Couldn't change permissions on the settings file!"
	echo "Check permissions in $INSTALL_ROOT and re-run this script!"
	cleanup
	exit -1
fi

cleanup

echo "\nThe installation was successfull."
exit 0