<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: function_statistics.inc.php,v 1.5 2006/06/05 10:53:22 medea3 Exp $
*/

include 'inc/classes/Distribution.php';

function get_average($array_of_numbers)
{
	$sum = 0;
	$nmb_of_elements = count($array_of_numbers);
	
	if ($nmb_of_elements == 0)
		return 0;

	if (!(is_array($array_of_numbers)))
		return $array_of_numbers[0];
	
	foreach($array_of_numbers AS $value)
		$sum += $value;
	
	return ($sum / $nmb_of_elements);
}

function get_median($array_of_numbers)
{
	$nmb_of_elements = count($array_of_numbers);
	
	if ($nmb_of_elements == 0)
		return 0;

	if ($nmb_of_elements == 1)
		return $array_of_numbers[0];
		
	sort($array_of_numbers);

	// Calculate median
	if( count($array_of_numbers) % 2 == 0 )
	{
		$median_index = count($array_of_numbers)/2-1;
		$q_median = ($array_of_numbers[$median_index] + $array_of_numbers[$median_index+1]) /2;
	}
	else
	{
		$median_index = count($array_of_numbers)/2 - 0.5;
		$q_median = $array_of_numbers[$median_index];
	}

	return $q_median;
}

function get_stddeviation($array_of_numbers)
{
	$nmb_of_elements = count($array_of_numbers);

	if ($nmb_of_elements < 2)
		return 0;

	$average = get_average($array_of_numbers);
	$deviation = 0;

	foreach ($array_of_numbers AS $value)
		$deviation += pow(($value - $average), 2);

	return sqrt ($deviation /= ( $nmb_of_elements - 1 ));
}

function get_quartiles($array_of_numbers)
{
	sort($array_of_numbers);
	
	$min = $array_of_numbers[0];
	$max = $array_of_numbers[count($array_of_numbers)-1];
	
	// Calculate median
	if( count($array_of_numbers) % 2 == 0 )
	{
		$median_index = count($array_of_numbers)/2-1;
		$q_median = ($array_of_numbers[$median_index] + $array_of_numbers[$median_index+1]) /2;
		$q1_upper = $median_index;
		$q3_lower = $median_index+1;
	}
	else
	{
		$median_index = count($array_of_numbers)/2 - 0.5;
		$q_median = $array_of_numbers[$median_index];
		$q1_upper = $median_index;
		$q3_lower = $median_index;
	}
	// Calculate quarters
	// unequal number of elements
	if( $q1_upper % 2 == 0)
	{
		$median_index = $q1_upper/2;
		$q_lower25 = $array_of_numbers[$median_index];
		$q_upper25 = $array_of_numbers[$q3_lower+$median_index];
	}
	else
	{
		$median_index = floor($q1_upper/2);
		$q_lower25 = ($array_of_numbers[$median_index] + $array_of_numbers[$median_index+1])/2;
		$q_upper25 = ($array_of_numbers[$q3_lower+$median_index] + $array_of_numbers[$q3_lower+$median_index+1])/2;
	}
	
	return array($min, $q_lower25, $q_median, $q_upper25, $max);
}

function anova_kw_test($categories, $significance_level)
{
	$ranking = anova_ranking($categories);

	$N = 0;
	$H_sum = 0;
	foreach($ranking AS $category)
	{
		$R = array_sum($category);
		$n = count($category);
		$N += $n;
		$H_sum += pow($R, 2)/$n;
	}
	$H = (12/($N*($N+1)))*$H_sum - 3*($N+1);

	$chi_distribution = new Distribution();
	$chi_square_table_value = $chi_distribution->getInverseChiSquare($significance_level, count($categories)-1);
	
	if($H<$chi_square_table_value)
		return true;
	else
		return false;
}

function anova_ranking($categories)
{
	$values = array();
	foreach($categories AS $category)
	{
		foreach($category AS $value)
		{
			$values[] = $value;
		}
	}
	sort($values);
	
	$rank = 1;
	$scoregroups = array();
	foreach($values AS $score)
	{
		$scoregroups[$score]++;
	}
	foreach($scoregroups AS $value => $counts)
	{
		$tmprank = 0;
		for($i=0;$i<$counts;$i++)
		{
			$tmprank += $rank++;
		}
		$ranking[$value] = $tmprank/$counts;
	}

	foreach($categories AS $cat_id => $category)
	{
		foreach($category AS $value_id => $value)
		{
			$categories[$cat_id][$value_id] = $ranking[$categories[$cat_id][$value_id]];
		}
	}
	return $categories;
}

function generate_histogram_data($data = array())
{
	if(count($data) > 0)
	{
		$min_value = min($data);
		$min_value--;
		$max_value  = max($data);
	
		$decrese_min = false;
		while((($max_value - $min_value) % 10) != 0)
		{
			if($decrese_min)
			{
				$min_value--;
				$decrese_min = false;
			}
			else
			{
				$max_value++;
				$decrese_min = true;
			}
			
		}
		$interval = ($max_value-$min_value)/10;
	
		for($i=$min_value;$i<$max_value;$i+=$interval)
		{
			$return_array[$i.'-'.($i+$interval)]['min'] = $i;
			$return_array[$i.'-'.($i+$interval)]['max'] = $i+$interval;
	
			foreach($data AS $id => $value)
			{
				if($value > $i && $value <= ($i+$interval))
				{
					$return_array[$i.'-'.($i+$interval)]['count']++;
					unset($data[$id]);
				}
			}
		}
	}
	
	return $return_array;
}

?>