<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: report_answers_by_category.inc.php,v 1.13 2006/06/01 12:32:10 medea2 Exp $
*/

if ($print_file)
{
	$pdf->print_h2('Answers to graded questions by category');
	if ( $nmb_of_categories > 0 )
	{
		$pdf->Write(5, 'The questions are ordered by category. Questions in ');
		$pdf->SetFont('', 'B');
		$pdf->Write(5, 'bold');
		$pdf->SetFont('');
		$pdf->Write(5, ' are ');
		$pdf->SetFont('', 'I');
		$pdf->Write(5, 'negated.');
		$pdf->SetFont('');
		$pdf->Ln(10);
	
		foreach($category_list AS $id => $category)
		{
			$cat_name = $category['name'];
			$cat_score = number_format($category['score'], 1, ".", "");
			$pdf->print_h3($cat_name . ' (' . $cat_score . ')');
			
			if (count($category['questions']) > 0)
			{
				$header = array('Question', 'Avg. score', 'Median', 'Std. deviation');
				$width = array(118, 15, 17, 20);
				$align = array('L', 'R', 'R', 'R');
				unset($data);
				unset($formatting);
				
				foreach($category['questions'] AS $q_id)
				{
					$q_negate = $questions[$q_id]['neg'];
					$q_order = $questions[$q_id]['order'];
					$q_name = $questions[$q_id]['name'];
					$rounded_stdd = number_format($questions[$q_id]['stdd'], 1, ".", "");
					$rounded_median = number_format($questions[$q_id]['median'], 1, ".", "");
		
					if ($q_negate)
					{
						$rounded_score = number_format((-1*$questions[$q_id]['avg']), 1, ".", "");
						$formatting[] = 'B';
					}
					else
					{
						$rounded_score = number_format($questions[$q_id]['avg'], 1, ".", "");
						$formatting[] = '';
					}
					$data[] = array($q_order.". ". $q_name, $rounded_score, $rounded_median, $rounded_stdd);
				
				}
				$pdf->FancyTable($header,$data,$width,$align,$formatting,array($row_color1, $row_color2));
			}
			else
				$pdf->Row(array("No questions in this category!\n\n"), 75, 'L', 0, '');
		
		}
	}
	else
	{
		$pdf->write(5, 'No categories in this questionnaire!');
		$pdf->Ln(10);
	}
}
else
{
if ( $nmb_of_categories > 0 )
{
?>
<p>The questions are ordered by category. Questions in <b>bold</b> are <i>negated.</i></p>
<?php
	foreach($category_list AS $id => $category)
	{
		$cat_name = $category['name'];
		$cat_score = number_format($category['score'], 1, ".", "");
		echo "<h3>$cat_name ($cat_score)</h3>\n";
		
		if (count($category['questions']) > 0)
		{
			echo "<table>\n";
			echo "<tr bgcolor=\"#e0e0e0\"><td class=\"report_answes_question\"><b>Question</b></td><td class=\"report_avg_median\" align=\"right\"><b>Avg. score</b></td><td class=\"report_avg_median\" align=\"right\"><b>Median</b></td><td class=\"report_pc\" align=\"right\"><b>Std. deviation</b></td></tr>\n";
	
			$row_color = $row_color2;
			foreach($category['questions'] AS $q_id)
			{
				$q_negate = $questions[$q_id]['neg'];
				$q_order = $questions[$q_id]['order'];
				$q_name = $questions[$q_id]['name'];
				$rounded_stdd = number_format($questions[$q_id]['stdd'], 1, ".", "");
				$rounded_median = number_format($questions[$q_id]['median'], 1, ".", "");
	
				// Alternating row color
				if ($row_color == $row_color2)
					$row_color = $row_color1;
				else
					$row_color = $row_color2;
	
				if ($q_negate)
				{
					$rounded_score = number_format((-1*$questions[$q_id]['avg']), 1, ".", "");
					echo "<tr bgcolor=\"$row_color\"><td><b>$q_order. $q_name</b></td><td align=\"right\"><b>$rounded_score</b></td><td align=\"right\"><b>$rounded_median</b></td><td align=\"right\"><b>$rounded_stdd</b></td></tr>\n";
				}
				else
				{
					$rounded_score = number_format($questions[$q_id]['avg'], 1, ".", "");
					echo "<tr bgcolor=\"$row_color\"><td>$q_order. $q_name</td><td align=\"right\">$rounded_score</td><td align=\"right\">$rounded_median</td><td align=\"right\">$rounded_stdd</td></tr>\n";
				}
			}
			echo "</table>\n";
		}
		else
			echo "No questions in this category!";
	}
}
else
	echo 'No categories in this questionnaire!';
}
?>