<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: report_category_boxplot.inc.php,v 1.14 2006/06/01 12:30:41 medea2 Exp $
*/

if($print_file)
{
	$pdf->print_h2('Categories');
	
	if ( $nmb_of_categories > 0 )
	{
		$pdf->print_h3('Summary');

		$header = array('Category', 'Avg. score', 'Median', 'Std. deviation');
		$width = array(118, 15, 17, 20);
		$align = array('L', 'R', 'R', 'R');
		unset($data);
		foreach($category_list AS $id => $category)
		{
			$cat_name = $category['name'];
			$cat_score = number_format($category['score'], 1, ".", "");
			$cat_median = number_format($category['median'], 1, ".", "");
			$cat_stdd = number_format($category['stdd'], 1, ".", "");
				
			$data[] = array($cat_name, $cat_score, $cat_median, $cat_stdd);
		}
		$pdf->FancyTable($header,$data,$width,$align,$formatting,array($row_color1, $row_color2));
	
		$pdf->print_h3('Differences based on satisfaction grade of master question.');
	
		if($master_question == 0 || $masterq_answers == 0)
			$pdf->Write(5, 'Questionnaire contains no master question');
		else
		{
	
			foreach($category_list AS $id => $category)
			{
				$cat_name = $category['name'];
				$cat_score = number_format($category['score'], 1, ".", "");
				
				$addtext = array();
			
				if (count($category['questions']) > 0)
				{
					foreach($category['groups'] AS $group => $values)
					{
						$argument[$masterq[$group]['label'].' ('.count($values['answers']).')'] = $values['quartiles'];
					}
					
					$src = return_multiboxplot_link("$cat_name ($cat_score)", $argument);
			
					$src = str_replace(' ', '%20', $src);
					$src = str_replace('&amp;', '&', $src);
			
					// Fetch working dir
					preg_match('/(.+)\/[^\/]+$/', $_SERVER["SCRIPT_NAME"], $matches);
					$href = 'http://localhost'.$matches[1].'/'.$src;
					
					// Get image height
					$im = @imagecreatefrompng($href);
					$imheight = imagesy($im);
					
					$imheight = ($imheight/72)*25.4;
					$pdf->CheckPageBreak($imheight);
					
					$pdf->Image($href, $pdf->GetX()+80, $pdf->GetY(), 90, 0, 'PNG');
	
					$invalid_comparefound = false;
					foreach($category['groups'] AS $key => $value)
					{
						if(!$value['comparable'])
						{
							$addtext[] = 'Comparisons with '.$value['label'].' not possible'."\n";
							$invalid_compare_found = true;
						}
					}
					if(!$invalid_compare_found)
						$addtext[] = 'Comparisons with all the groups are possible'."\n";
	
					$addtext[] = "\n";
					$differsfound = false;
					foreach($category['groups'] AS $key => $value)
					{
						if(count($value['differs']))
						{
							$differsfound = true;
							foreach($value['differs'] AS $differ)
							{
								$addtext[] = $masterq[$key]['label'] . ' differs from ' . $masterq[$differ]['label'] . "\n";
							}
						}
					}
					if(!$differsfound)
						$addtext[] = 'No differences found'."\n";
				}
				else
					$addtext[] = "No questions in this category!";
				
	
				$pdf->print_h4($cat_name . ' ('. $cat_score .')');
	
				if (count($category['questions']) > 0)
				{
					$pdf->SetFont('Times','B',10);
					$pdf->Cell(0, 4, 'Statistical analysis', 0, 1);
					$pdf->SetFont('');
				}
				
				$addtext[] = "\n";
				
				$height = 20;
				foreach($addtext AS $text)
				{
					$height += $pdf->Row(array($text), 75, 'L', 0, '');
				}
				if($height < $imheight)
					$pdf->Ln($imheight-$height);
				
				$pdf->print_normal();
			}
		}
	}
	else
	{
		$pdf->write(5, 'No categories in this questionnaire!');
		$pdf->Ln(10);
	}
}
else
{
if ( $nmb_of_categories > 0 )
{
?>
	<h3>Summary</h3>
	<table>
	<tr bgcolor="#e0e0e0">
		<td class="report_answes_question"><b>Category</b></td>
		<td class="report_avg_median" align="right"><b>Avg. score</b></td>
		<td class="report_avg_median" align="right"><b>Median</b></td>
		<td align="right" class="report_pc"><b>Std. deviation</b></td>
	</tr>
	<?php
		$row_color = $row_color2;
		foreach($category_list AS $id => $category)
		{
			$cat_name = $category['name'];
			$cat_score = number_format($category['score'], 1, ".", "");
			$cat_median = number_format($category['median'], 1, ".", "");
			$cat_stdd = number_format($category['stdd'], 1, ".", "");
			
			// Alternating row color
			if ($row_color == $row_color2)
				$row_color = $row_color1;
			else
				$row_color = $row_color2;
		
			echo "<tr bgcolor=\"$row_color\"><td>$cat_name</td><td align=\"right\">$cat_score</td><td align=\"right\">$cat_median</td><td align=\"right\">$cat_stdd</td></tr>";
		}
	?>
	</table>
	<h3>Differences based on satisfaction grade of master question.</h3>
	<?php
	if($master_question == 0 || $masterq_answers == 0)
		echo "Questionnaire contains no master question";
	else
	{
		$bgcolor = "#F0F0F0";
		echo "<table cellspacing=\"0\" cellpadding=\"0\">";
		foreach($category_list AS $id => $category)
		{
			$cat_name = $category['name'];
			$cat_score = number_format($category['score'], 1, ".", "");
			echo "<tr><td colspan=\"2\"><h4>$cat_name ($cat_score)</h4></td></tr>\n";
			if (count($category['questions']) > 0)
			{
	?>
				<tr>
					<td class="report_plot_td" valign="top" bgcolor="<?php echo $bgcolor ?>">
						<b>Statistical analysis</b>
					</td>
					<td rowspan="4" bgcolor="<?php echo $bgcolor ?>">
	<?php	
					foreach($category['groups'] AS $group => $values)
					{
						$argument[$masterq[$group]['label'].' ('.count($values['answers']).')'] = $values['quartiles'];
					}
					print_multiboxplot("$cat_name ($cat_score)", $argument);	
	?>
					</td>
				</tr>
				<tr>
				<td valign="top" bgcolor="<?php echo $bgcolor ?>">
	<?php
					$invalid_comparefound = false;
					foreach($category['groups'] AS $key => $value)
					{
						if(!$value['comparable'])
						{
							echo('Comparisons with '.$value['label'].' not possible<br />');
							$invalid_compare_found = true;
						}
					}
					if(!$invalid_compare_found)
						echo('Comparisons with all the groups are possible');
					
					echo "</td></tr>\n";
					echo "<tr><td bgcolor=\"$bgcolor\"><hr /></td></tr>\n";
					echo "<tr><td bgcolor=\"$bgcolor\" valign=\"top\">";
					$differsfound = false;
					foreach($category['groups'] AS $key => $value)
					{
						if(count($value['differs']))
						{
							$differsfound = true;
							foreach($value['differs'] AS $differ)
							{
								echo($masterq[$key]['label'] . ' differs from ' . $masterq[$differ]['label'] . '<br />');
							}
						}
					}
					if(!$differsfound)
						echo('No differences found<br />');
?>
					</td></tr>
<?php
			}
			else
				echo '<tr><td colspan="2">No questions in this category!</td></tr>';
		}
?>
	</table>
<?php
	}
}
else
	echo 'No categories in this questionnaire!';
}
?>