<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: show_report_pdf.inc.php,v 1.5 2006/06/02 08:55:57 medea8 Exp $
*/

require('inc/classes/FPDF/fpdf.php');

class PDF extends FPDF
{
	var $aligns;

	//Page header
	function Header()
	{
		//Logo
		$this->Image('img/zeus_logo_tiny.png',10,8);
		//Arial bold 15
		$this->SetFont('Arial','B',15);
		//Move to the right
		$this->Cell(40);
		//Title
		$this->MultiCell(120,10,$this->title,0,'C');
		//Line break
		$this->Ln(20);
	}

	//Page footer
	function Footer()
	{
		//Position at 1.5 cm from bottom
		$this->SetY(-20);
		//Arial italic 8
		$this->SetFont('Arial','I',8);
		//Page number
		$this->Cell(0,6,'Page '.$this->PageNo().'/{nb}',0,1,'C');
		// Set timestamp
		$this->Cell(0,6,'Report generated: '.date('r'),0,0,'C');
	}
	
	//Colored table
	function FancyTable($header,$data,$width,$align,$rowformatting=array(), $rowcolors=array('#E2EFFC', '#FFFFFF'))
	{
		//Colors, line width and bold font
		$this->SetFillColor(224,224,224);
		$this->SetTextColor(0);
		$this->SetDrawColor(255,255,255);
		$this->SetLineWidth(.1);
		$this->SetFont('','B');
		
		//Header
		if(count($header) > 0)
		{
			$this->Row($header, $width, $align, 1, 'LR');
		}
		
		//Color and font restoration
		//$this->SetFillColor(226,239,252);
		$this->SetTextColor(0);
		$this->SetFont('');
		
		//Data
		if(is_array($data))
		{
			foreach($data as $id => $row)
			{
				if($fill)
					$this->SetFillColor_from_hex($rowcolors[1]);
				else
					$this->SetFillColor_from_hex($rowcolors[0]);
	
				if(!empty($rowformatting[$id]))
					$this->SetFont('', $rowformatting[$id]);
	
				if(!is_array($row) && ($row == 1))
				{
					$this->SetDrawColor(0,0,0);
					$this->Cell(array_sum($width),1,'','T');
					$this->Ln();
					$this->SetDrawColor(255,255,255);
					$fill=!$fill;
				}
				else
				{
					$this->Row($row, $width, $align, 1, 'LR');
				}
				$this->SetFont('');
				$fill=!$fill;
			}
		}
		$this->Ln();
	}
	function SetWidths($w)
	{
		//Set the array of column widths
		$this->widths=$w;
	}

	function SetAligns($a)
	{
		//Set the array of column alignments
		$this->aligns=$a;
	}

	function Row($data, $width, $align, $fill, $border)
	{
		//Calculate the height of the row
		$nb=0;
		for($i=0;$i<count($data);$i++)
		{
			$nb_indv[$i] = $this->NbLines($width[$i],$data[$i]);
			$nb=max($nb,$nb_indv[$i]);
		}
		$h=5*$nb;
		//Issue a page break first if needed
		$this->CheckPageBreak($h);
		//Draw the cells of the row
		for($i=0;$i<count($data);$i++)
		{
			// Add newlines to small rows
			for($j=$nb_indv[$i];$j<=$nb;$j++)
				$data[$i] .= "\n";
			$w=$width[$i];
			$a=isset($align[$i]) ? $align[$i] : 'L';
			//Save the current position
			$x=$this->GetX();
			$y=$this->GetY();
			//Draw the border
			$this->Rect($x,$y,$w,$h);
			//Print the text
			$this->MultiCell($w,5,$data[$i],$border,$a, $fill);
			//Put the position to the right of the cell
			$this->SetXY($x+$w,$y);
		}
		//Go to the next line
		$this->Ln($h);
		return $h;
	}

	function CheckPageBreak($h)
	{
		//If the height h would cause an overflow, add a new page immediately
		if($this->GetY()+$h>$this->PageBreakTrigger)
			$this->AddPage($this->CurOrientation);
	}

	function NbLines($w,$txt)
	{
		//Computes the number of lines a MultiCell of width w will take
		$cw=&$this->CurrentFont['cw'];
		if($w==0)
		$w=$this->w-$this->rMargin-$this->x;
		$wmax=($w-2*$this->cMargin)*1000/$this->FontSize;
		$s=str_replace("\r",'',$txt);
		$nb=strlen($s);
		if($nb>0 and $s[$nb-1]=="\n")
		$nb--;
		$sep=-1;
		$i=0;
		$j=0;
		$l=0;
		$nl=1;
		while($i<$nb)
		{
			$c=$s[$i];
			if($c=="\n")
			{
				$i++;
				$sep=-1;
				$j=$i;
				$l=0;
				$nl++;
				continue;
			}
			if($c==' ')
				$sep=$i;
			$l+=$cw[$c];
			if($l>$wmax)
			{
				if($sep==-1)
				{
					if($i==$j)
						$i++;
				}
				else
					$i=$sep+1;
				$sep=-1;
				$j=$i;
				$l=0;
				$nl++;
			}
			else
				$i++;
		}
		return $nl;
	}

	function print_h2($text)
	{
		$this->SetFont('Times','B',16);	
		$this->Cell(0, 6, $text, 0, 1);
		$this->print_normal();
		$this->Ln();
	}
	function print_h3($text)
	{
		$this->SetFont('Times','B',14);	
		$this->Cell(0, 6, $text, 0, 1);
		$this->print_normal();
		$this->Ln();
	}
	function print_h4($text)
	{
		$this->SetFont('Times','B',12);	
		$this->Cell(0, 4, $text, 0, 1);
		$this->print_normal();
		$this->Ln();
	}
	
	function print_normal()
	{
		$this->SetFont('Times', '', 12);
	}
	function setFillColor_from_hex($hex)
	{
		$r = hexdec(substr($hex, 1, 2));
		$g = hexdec(substr($hex, 3, 2));
		$b = hexdec(substr($hex, 5, 2));
		
		$this->SetFillColor($r,$g,$b);
	}


}

//Instanciation of inherited class
$pdf=new PDF();
$pdf->AliasNbPages();
$pdf->SetFont('Times','',12);
$pdf->SetLeftMargin(20);