<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: create_bar_plot.php,v 1.1 2006/05/22 13:07:03 medea3 Exp $
*/

include 'create_aligned_text.inc.php';

function countdigits($num)
{
	$count = 0;
	if($num)
	{
		if($num < 1)
		{
			while($num < 1)
			{
				$count++;
				$num *= 10;
			}
			$count *= -1;
		}
		else
		{
			while ($num > 1)
			{	$count++;
				$num /= 10;
			}
		}
	}
	return $count;
}

function max_label_value($num)
{
	$length = countdigits($num);
	if($length == 1)
		$length = 2;
	elseif($length < 0)
		$length += 2;
	$quote = $num/(pow(10, $length-1));
	$num = ceil($quote);
	$num = $num*(pow(10, $length-1));
	return $num;
}

header("Content-type: image/png");

$top_padding = 35;
$bottom_padding = 35;
$left_padding = 35;
$right_padding = 35;
$bar_width = 15;
$bar_space_width = 10;

$box_height = 200;
$box_width = ($bar_width+$bar_space_width)*count($_REQUEST['value'])+$bar_space_width;

$space_to_arrow = 12;
$axis_scale_line = 3;
$nmb_of_scale_lines = 10;

$max_value = max($_REQUEST['value']);
$max_label = max_label_value($max_value);

$left_padding += countdigits($max_label) * 5;

$width = $left_padding + $right_padding + $box_width;
$height = $top_padding + $bottom_padding + $box_height;

$img = imagecreatetruecolor($width, $height)
    or die("Cannot Initialize new GD image stream");

// allocate colors
$white = imagecolorallocate($img, 255, 255, 255);
$black = imagecolorallocate($img, 0, 0, 0);
$background_color = imagecolorallocate($img, 240, 240, 240);
$grey = imagecolorallocate($img, 210, 210, 210);
$box_color = imagecolorallocatealpha($img, 110, 169, 227, 20);
$red_color = imagecolorallocate($img, 255, 0, 0);

// Styled image
$dashed_line = array($red_color, $red_color, $red_color, $background_color, $background_color, $background_color);

// Fonts
$fontfile = "./Vera.ttf";

// Set background-color
imagefill($img, 0, 0, $background_color);

// Print title
$title = $_REQUEST['title'];
$fontsize = 10;
imageTextWrapped(&$img, 0, 10, $width, $fontfile, $black, $title, $fontsize, "c");

// Master label, only works with reasonable fontsizes
$fontsize=7;

// Print y axis
imageline($img, $left_padding, $top_padding, $left_padding, $height-$bottom_padding, $black);
imageline($img, $left_padding, $top_padding-1, $left_padding+4, $top_padding+3, $black);
imageline($img, $left_padding, $top_padding-1, $left_padding-4, $top_padding+3, $black);

// Print y label
$y_label = $_REQUEST['y_label'];
imagettftext($img, 10, 90, 15, $height/2+strlen($y_label)*4, $black, $fontfile, $y_label);

// Print y scale
for ($i = 0; $i < $nmb_of_scale_lines; $i++)
{
	$y_position = $top_padding + $space_to_arrow + $i * (($box_height - $space_to_arrow) / $nmb_of_scale_lines);
	$x1 = $left_padding-$axis_scale_line;
	$y1 = $y_position;
	$x2 = $left_padding;
	$y2 = $y_position;
	
	$text = $max_label-($max_label*$i/$nmb_of_scale_lines);
	
	imageline($img, $x1, $y1, $x2, $y2, $black);
	imageline($img, $left_padding+1, $y1, $width-$right_padding, $y2, $grey);
	imageTextWrapped(&$img, 0, $y_position - ($fontsize / 2), $left_padding, $fontfile, $black, $text, $fontsize, "r");
}

// Print x axis
imageline($img, $left_padding+1, $height-$bottom_padding, $width-$right_padding, $height-$bottom_padding, $black);
imageline($img, $width-$right_padding-3, $height-$bottom_padding-4, $width-$right_padding, $height-$bottom_padding, $black);
imageline($img, $width-$right_padding-3, $height-$bottom_padding+4, $width-$right_padding, $height-$bottom_padding, $black);

// Print x label
$x_label = $_REQUEST['x_label'];
imagettftext($img, 10, 0, $left_padding+$box_width/2-strlen($x_label)*4, $height-5, $black, $fontfile, $x_label);

// Print bars
for($i=0;$i<count($_REQUEST['value']);$i++)
{
	$x = $left_padding+$bar_space_width+($bar_space_width+$bar_width)*$i;
	$y = $top_padding + $space_to_arrow + ($box_height - $space_to_arrow)-(($box_height - $space_to_arrow)*($_REQUEST['value'][$i]/$max_label));
	
	$x1 = $x+($bar_width/2);
	$x2 = $x1;
	$y1 = $height-$bottom_padding;
	$y2 = $y1+$axis_scale_line;
	
	if($_REQUEST['origo'] == $_REQUEST['labels'][$i])
	{
		imagesetstyle($img, $dashed_line);
		imageline($img, $x1, $top_padding, $x2, $y2, IMG_COLOR_STYLED);
	}

	imagefilledrectangle($img, $x, $y, $x+$bar_width, $height-$bottom_padding, $box_color);
	imagerectangle($img, $x, $y, $x+$bar_width, $height-$bottom_padding, $black);
	
	imageline($img, $x1, $y1, $x2, $y2, $black);
	imageTextWrapped(&$img, $x, $height-$bottom_padding+7, $bar_space_width+$bar_width, $fontfile, $black, $_REQUEST['labels'][$i], $fontsize, "c");
}



imagepng($img);
imagedestroy($img);

?>