<?php
/*
Copyright (c) 2006, University of Karlstad
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

$Id: create_piechart.php,v 1.1 2006/06/02 08:30:09 medea8 Exp $
*/

// Set properties
// image
$width = 500;
$height = 300;
// margins
$topmargin = 20;
// circle
$radius = 100;
$center = array(150, $height/2+$topmargin);
$elipsefactor = 1;
// fonts (should be a valid ttf-file)
$fontfile = "./Vera.ttf";

include 'create_aligned_text.inc.php';

// Functions
function polar2rect($radius, $angle)
{
	$rad = deg2rad($angle);
	$x = cos($rad)*$radius;
	$y = sin($rad)*$radius;
	return array($x, $y);
}

function printPercentLabel($angle, $percent)
{
	global $radius, $center, $elipsefactor, $img, $fontfile;
	list($text_x, $text_y) = polar2rect($radius+10, $angle);
	$text_y *= $elipsefactor;

	if($text_x < 0)
		$text_x += cos(deg2rad($angle))*40;
	if($text_y > 0)
		$text_y += sin(deg2rad($angle))*10;

	$text_x += $center[0];
	$text_y += $center[1];

	imagettftext($img, 10, 0, $text_x, $text_y, 0, $fontfile, number_format($percent, 1, '.', '').'%');
}

function printLabel($label, $color, $order)
{
	global $radius, $center, $img, $fontfile, $topmargin;

	$x = $center[0]+$radius+70;
	$y = $order*24+$topmargin+20;

	imagefilledrectangle($img, $x, $y, $x+12, $y+12, $color);
	imagerectangle($img, $x, $y, $x+12, $y+12, 0);
	
	imagettftext($img, 10, 0, $x+20, $y+12, 0, $fontfile, $label);
}

// create image
$img = imagecreatetruecolor($width, $height);

// allocate colors
$white = imagecolorallocate($img, 255, 255, 255);
$black = imagecolorallocate($img, 0, 0, 0);

// starting angle
if(empty($_REQUEST['startangle']))
	$startangle = 270;
else
	$startangle = $_REQUEST['startangle'];

// set vars from $_REQUEST
$labels = array();
$percents = array();
if(is_array($_REQUEST['labels'])) $labels = $_REQUEST['labels'];
if(is_array($_REQUEST['percents'])) $percents = $_REQUEST['percents'];

//Random colors.
for($i=0;$i<=count($labels);$i++){
   $r=rand(100,255);$g=rand(100,255);$b=rand(100,255);
   $colors[$i] = imagecolorallocate($img,$r,$g,$b);
   $colord[$i] = imagecolorallocate($img,($r/2),($g/2),($b/2));
}

// background
imagefill($img, 0, 0, $white);

// Print title
$fontsize=12;
imageTextWrapped(&$img, $width/2, $fontsize/2, strlen($_REQUEST['title']), $fontfile, $black, $_REQUEST['title'], $fontsize, $align="c");

// loop through parts
$totalpercent = 0.0;
$i = 0;
foreach($labels AS $key => $label)
{
	if($percents[$key] > 0 && $totalpercent+$percents[$key]<=100)
	{
		$drawingangle = 3.6*$percents[$key];
		imagefilledarc($img, $center[0], $center[1], $radius*2, $radius*2*$elipsefactor, $startangle, $startangle+$drawingangle, $colors[$i], IMG_ARC_PIE);
		imagefilledarc($img, $center[0], $center[1], $radius*2, $radius*2*$elipsefactor, $startangle, $startangle+$drawingangle, $black, IMG_ARC_NOFILL | IMG_ARC_EDGED);

		// Text
		$textangle = $startangle+($drawingangle/2);
		printPercentLabel($textangle, $percents[$key]);
		
		printLabel($label, $colors[$i], $i);

		$startangle += $drawingangle;

		$totalpercent += $percents[$key];
		$i++;
	}
}

// Fill unknown slice
if($totalpercent != 100)
{
	$percent = 100-$totalpercent;
	$drawingangle = 3.6*$percent;
	imagefilledarc($img, $center[0], $center[1], $radius*2, $radius*2*$elipsefactor, $startangle, $startangle+$drawingangle, $black, IMG_ARC_NOFILL | IMG_ARC_EDGED);
	
	// Text
	$textangle = $startangle+($drawingangle/2);
	printPercentLabel($textangle, $percent);
	
	printLabel('unknown', $white, $i);
}

// output image browser
header("Content-type: image/png");
imagepng($img);

// free memory
imagedestroy($img);

?>
